@@ -1,8 +1,10 @@
 /*
- * Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (c) 2015-2024 Google, Inc.
+ * Copyright (c) 2015-2026 The Khronos Group Inc.
+ * Copyright (c) 2015-2026 Valve Corporation
+ * Copyright (c) 2015-2026 LunarG, Inc.
+ * Copyright (c) 2015-2026 Google, Inc.
+ * Copyright (C) 2025 Arm Limited.
+ * Modifications Copyright (C) 2025-2026 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,23 +24,21 @@
 #include <android_native_app_glue.h>
 #endif
 
-#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
-#include "wayland-client.h"
-#endif
-
 #include <vulkan/utility/vk_format_utils.h>
 #include <vulkan/utility/vk_struct_helper.hpp>
 
+// Remove Windows macro that prevents usage of its name in any scope of the program.
+// For example, BitstreamBuffer::MemoryBarrier() won't compile on ARM64.
+#if defined(VK_USE_PLATFORM_WIN32_KHR) && defined(MemoryBarrier)
+#undef MemoryBarrier
+#endif
+
 #include "binding.h"
-#include "containers/custom_containers.h"
-#include "generated/vk_extension_helper.h"
+#include "containers/limits.h"
 #include "render.h"
 
-#include <cmath>
-#include <functional>
-#include <limits>
-#include <string>
 #include <vector>
+#include <array> // Required for Windows in many tests
 
 // MSVC and GCC define __SANITIZE_ADDRESS__ when compiling with address sanitization
 // However, clang doesn't. Instead you have to use __has_feature to check.
@@ -70,16 +70,17 @@
 
 #define OBJECT_LAYER_NAME "VK_LAYER_KHRONOS_validation"
 
-//--------------------------------------------------------------------------------------
-// Mesh and VertexFormat Data
-//--------------------------------------------------------------------------------------
+[[maybe_unused]] static const VkBool32 kVkFalse = VK_FALSE;
+[[maybe_unused]] static const VkBool32 kVkTrue = VK_TRUE;
+
+// This is only for tests where you have a good reason to have more than the default (10) duplicate message limit.
+// It is highly suggested you first try to breakup your test up into smaller tests if you are trying to use this.
+static const VkLayerSettingEXT kDisableMessageLimitSetting = {OBJECT_LAYER_NAME, "enable_message_limit",
+                                                              VK_LAYER_SETTING_TYPE_BOOL32_EXT, 1, &kVkFalse};
+[[maybe_unused]] static VkLayerSettingsCreateInfoEXT kDisableMessageLimit = {VK_STRUCTURE_TYPE_LAYER_SETTINGS_CREATE_INFO_EXT,
+                                                                             nullptr, 1, &kDisableMessageLimitSetting};
 
 // Static arrays helper
-template <class ElementT, size_t array_size>
-size_t size(ElementT (&)[array_size]) {
-    return array_size;
-}
-
 template <class ElementT, size_t array_size>
 uint32_t size32(ElementT (&)[array_size]) {
     return static_cast<uint32_t>(array_size);
@@ -91,28 +92,31 @@ uint32_t size32(const Container &c) {
 }
 
 // Format search helper
-VkFormat FindSupportedDepthOnlyFormat(VkPhysicalDevice phy);
-VkFormat FindSupportedStencilOnlyFormat(VkPhysicalDevice phy);
-VkFormat FindSupportedDepthStencilFormat(VkPhysicalDevice phy);
+VkFormat FindSupportedDepthOnlyFormat(VkPhysicalDevice gpu);
+VkFormat FindSupportedStencilOnlyFormat(VkPhysicalDevice gpu);
+VkFormat FindSupportedDepthStencilFormat(VkPhysicalDevice gpu);
 
 // Returns true if *any* requested features are available.
 // Assumption is that the framework can successfully create an image as
 // long as at least one of the feature bits is present (excepting VTX_BUF).
-bool FormatIsSupported(VkPhysicalDevice phy, VkFormat format, VkImageTiling tiling = VK_IMAGE_TILING_OPTIMAL,
+bool FormatIsSupported(VkPhysicalDevice gpu, VkFormat format, VkImageTiling tiling = VK_IMAGE_TILING_OPTIMAL,
                        VkFormatFeatureFlags features = ~VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT);
 
 // Returns true if format and *all* requested features are available.
-bool FormatFeaturesAreSupported(VkPhysicalDevice phy, VkFormat format, VkImageTiling tiling, VkFormatFeatureFlags features);
+bool FormatFeaturesAreSupported(VkPhysicalDevice gpu, VkFormat format, VkImageTiling tiling, VkFormatFeatureFlags features);
+bool FormatFeatures2AreSupported(VkPhysicalDevice gpu, VkFormat format, VkImageTiling tiling, VkFormatFeatureFlags2 features);
+
+// Small wrapprer around vkGetPhysicalDeviceImageFormatProperties
+VkResult GetImageFormatProps(VkPhysicalDevice gpu, const VkImageCreateInfo &ci, VkImageFormatProperties &out_limits);
 
 // Returns true if format and *all* requested features are available.
-bool ImageFormatIsSupported(const VkInstance inst, const VkPhysicalDevice phy, const VkImageCreateInfo info,
-                            const VkFormatFeatureFlags features);
+bool IsImageFormatSupported(VkPhysicalDevice gpu, const VkImageCreateInfo &ci, const VkFormatFeatureFlags features);
 
 // Returns true if format and *all* requested features are available.
-bool BufferFormatAndFeaturesSupported(VkPhysicalDevice phy, VkFormat format, VkFormatFeatureFlags features);
+bool BufferFormatAndFeaturesSupported(VkPhysicalDevice gpu, VkFormat format, VkFormatFeatureFlags features);
 
 // Simple sane SamplerCreateInfo boilerplate
-VkSamplerCreateInfo SafeSaneSamplerCreateInfo();
+VkSamplerCreateInfo SafeSaneSamplerCreateInfo(void *p_next = nullptr);
 
 // Dependent "false" type for the static assert, as GCC will evaluate
 // non-dependent static_asserts even for non-instantiated templates
@@ -120,21 +124,8 @@ template <typename T>
 struct AlwaysFalse : std::false_type {};
 
 // Helpers to get nearest greater or smaller value (of float) -- useful for testing the boundary cases of Vulkan limits
-template <typename T>
-T NearestGreater(const T from) {
-    using Lim = std::numeric_limits<T>;
-    const auto positive_direction = Lim::has_infinity ? Lim::infinity() : Lim::max();
-
-    return std::nextafter(from, positive_direction);
-}
-
-template <typename T>
-T NearestSmaller(const T from) {
-    using Lim = std::numeric_limits<T>;
-    const auto negative_direction = Lim::has_infinity ? -Lim::infinity() : Lim::lowest();
-
-    return std::nextafter(from, negative_direction);
-}
+float NearestGreater(const float from);
+float NearestSmaller(const float from);
 
 // Defining VVL_TESTS_USE_CUSTOM_TEST_FRAMEWORK allows downstream users
 // to inject custom test framework changes. This includes the ability
@@ -196,6 +187,20 @@ class VkLayerTest : public VkLayerTestBase {
     }
     APIVersion DeviceValidationVersion() const;
 
+    // Helpers to quickly create a Handle and check it gives a certain VU error
+    void CreateSamplerTest(const VkSamplerCreateInfo &create_info, const char *vuid);
+    void CreateBufferTest(const VkBufferCreateInfo &create_info, const char *vuid);
+    void CreateImageTest(const VkImageCreateInfo &create_info, const char *vuid);
+    void CreateBufferViewTest(const VkBufferViewCreateInfo &create_info, const char *vuid);
+    void CreateImageViewTest(const VkImageViewCreateInfo &create_info, const char *vuid);
+    void CreateRenderPassTest(const VkRenderPassCreateInfo &create_info, bool rp2_supported, const char *rp1_vuid,
+                              const char *rp2_vuid);
+    void CreateRenderPassBeginTest(const VkCommandBuffer command_buffer, const VkRenderPassBeginInfo *begin_info,
+                                   bool rp2_supported, const char *rp1_vuid, const char *rp2_vuid);
+
+    VkResolveModeFlagBits FindSupportedDepthResolveMode();
+    VkResolveModeFlagBits FindSupportedStencilResolveMode();
+
   protected:
     void SetTargetApiVersion(APIVersion target_api_version);
     bool LoadDeviceProfileLayer(
@@ -221,8 +226,8 @@ VkPhysicalDeviceProperties2 VkLayerTest::GetPhysicalDeviceProperties2(VkPhysical
 
 class VkBestPracticesLayerTest : public VkLayerTest {
   public:
-    void InitBestPracticesFramework(const char *ValidationChecksToEnable = "");
-    void InitBestPractices(const char *ValidationChecksToEnable = "");
+    void InitBestPracticesFramework(const char* vendor_checks_to_enable = nullptr);
+    void InitBestPractices(const char* vendor_checks_to_enable = nullptr);
 
   protected:
     VkValidationFeatureEnableEXT enables_[1] = {VK_VALIDATION_FEATURE_ENABLE_BEST_PRACTICES_EXT};
@@ -231,24 +236,58 @@ class VkBestPracticesLayerTest : public VkLayerTest {
 
 class GpuAVTest : public virtual VkLayerTest {
   public:
-    void InitGpuAvFramework(void *p_next = nullptr);
+    void InitGpuAvFramework(std::vector<VkLayerSettingEXT> layer_settings = {}, bool safe_mode = true);
 
     VkValidationFeaturesEXT GetGpuAvValidationFeatures();
 };
 
+class GpuAVGpuAVShaderSanitizer : public GpuAVTest {
+  public:
+    void SimpleZeroComputeTest(const char *shader, int source_type, const char *expected_error = nullptr, uint32_t error_count = 1);
+};
+
 class GpuAVBufferDeviceAddressTest : public GpuAVTest {
   public:
-    void InitGpuVUBufferDeviceAddress(void *p_next = nullptr);
+    void InitGpuVUBufferDeviceAddress(bool safe_mode = true);
 };
 
 class GpuAVDescriptorIndexingTest : public GpuAVTest {
   public:
-    void InitGpuVUDescriptorIndexing();
+    void InitGpuVUDescriptorIndexing(bool safe_mode = true);
+};
+
+class GpuAVMesh : public GpuAVTest {
+  public:
+    void InitBasicMeshAndTask(bool safe_mode = true);
+};
+
+class GpuAVDescriptorClassGeneralBuffer : public GpuAVTest {
+  public:
+    void ComputeStorageBufferTest(const char *shader, int source_type, VkDeviceSize buffer_size,
+                                  const char *expected_error = nullptr, uint32_t error_count = 1);
+};
+
+class GpuAVDescriptorClassGeneralBufferCoopMat : public GpuAVTest {
+  public:
+    void InitCooperativeMatrixKHR(bool safe_mode = true);
+    void BasicComputeTest(const char *shader, int source_type, VkDeviceSize buffer_size, const char *expected_error = nullptr,
+                          uint32_t error_count = 1);
+};
+
+class GpuAVCopyMemoryIndirect : public GpuAVTest {
+  public:
+    void InitGpuAVCopyMemoryIndirect(bool safe_mode = true);
 };
 
 class GpuAVRayQueryTest : public GpuAVTest {
   public:
-    void InitGpuAVRayQuery();
+    void InitGpuAVRayQuery(std::vector<VkLayerSettingEXT> layer_settings = {});
+};
+
+class GpuAVRayHitObjectTest : public GpuAVTest {
+  public:
+    void InitHitObjectMotionTest(std::vector<VkLayerSettingEXT> layer_settings = {});
+    void InitHitObjectTest(std::vector<VkLayerSettingEXT> layer_settings = {});
 };
 
 class DebugPrintfTests : public VkLayerTest {
@@ -256,23 +295,22 @@ class DebugPrintfTests : public VkLayerTest {
     void InitDebugPrintfFramework(void *p_next = nullptr, bool reserve_slot = false);
 };
 
-struct SyncValSettings;
-class VkSyncValTest : public VkLayerTest {
-  public:
-    void InitSyncValFramework(const SyncValSettings *p_sync_settings = nullptr);
-    void InitSyncVal(const SyncValSettings *p_sync_settings = nullptr);
-    void InitTimelineSemaphore();
-};
-
 class AndroidExternalResolveTest : public VkLayerTest {
   public:
     void InitBasicAndroidExternalResolve();
     bool nullColorAttachmentWithExternalFormatResolve;
 };
 
+class LegacyTest : public VkLayerTest {
+  public:
+    void CreateRenderPass();
+};
+
 class DescriptorBufferTest : public VkLayerTest {
   public:
-    void InitBasicDescriptorBuffer();
+    void InitBasicDescriptorBuffer(void *instance_pnext = nullptr);
+
+    VkPhysicalDeviceDescriptorBufferPropertiesEXT descriptor_buffer_properties = vku::InitStructHelper();
 };
 
 class DescriptorIndexingTest : public VkLayerTest {
@@ -280,10 +318,38 @@ class DescriptorIndexingTest : public VkLayerTest {
     void ComputePipelineShaderTest(const char *shader, std::vector<VkDescriptorSetLayoutBinding> &bindings);
 };
 
+class DescriptorHeapTest : public VkLayerTest {
+  public:
+    void InitBasicDescriptorHeap();
+    void CreateResourceHeap(VkDeviceSize app_size);
+    void CreateSamplerHeap(VkDeviceSize app_size, bool use_embedded_samplers = false);
+
+    void BindResourceHeap();
+    void BindSamplerHeap();
+
+    VkDeviceSize AlignedAppend(VkDeviceSize& end, VkDescriptorType type, uint32_t count = 1);
+    VkDeviceSize AlignResource(VkDeviceSize offset);
+    VkDeviceSize AlignSampler(VkDeviceSize offset);
+
+    VkPhysicalDeviceDescriptorHeapPropertiesEXT heap_props = vku::InitStructHelper();
+
+    vkt::Buffer resource_heap_;
+    uint8_t* resource_heap_data_ = nullptr;
+    vkt::Buffer sampler_heap_;
+    uint8_t* sampler_heap_data_ = nullptr;
+
+  private:
+    bool embedded_samplers = false;
+};
+
 class DynamicRenderingTest : public VkLayerTest {
   public:
     void InitBasicDynamicRendering();
     void InitBasicDynamicRenderingLocalRead();
+
+    VkRenderingInfo GetSimpleRenderingInfo();
+    VkRenderingInfo GetSimpleSuspendInfo();
+    VkRenderingInfo GetSimpleResumeInfo();
 };
 
 class DynamicStateTest : public VkLayerTest {
@@ -303,6 +369,9 @@ class ExternalMemorySyncTest : public VkLayerTest {
 class DeviceGeneratedCommandsTest : public VkLayerTest {
   public:
     void InitBasicDeviceGeneratedCommands();
+
+    void SetPreProcessBuffer(VkGeneratedCommandsInfoEXT &generated_commands_info, void *pipeline_or_shader_object = nullptr);
+    std::unique_ptr<vkt::Buffer> pre_process_buffer_ = std::make_unique<vkt::Buffer>();
 };
 
 class GraphicsLibraryTest : public VkLayerTest {
@@ -310,13 +379,15 @@ class GraphicsLibraryTest : public VkLayerTest {
     void InitBasicGraphicsLibrary();
 };
 
+class TileMemoryHeapTest : public VkLayerTest {
+  public:
+};
+
 class HostImageCopyTest : public VkLayerTest {
   public:
-    void InitHostImageCopyTest(const VkImageCreateInfo &create_info);
+    void InitHostImageCopyTest();
     bool CopyLayoutSupported(const std::vector<VkImageLayout> &copy_src_layouts, const std::vector<VkImageLayout> &copy_dst_layouts,
                              VkImageLayout layout);
-    VkFormat compressed_format = VK_FORMAT_UNDEFINED;
-    bool separate_depth_stencil = false;
     std::vector<VkImageLayout> copy_src_layouts;
     std::vector<VkImageLayout> copy_dst_layouts;
 
@@ -338,6 +409,11 @@ class ImageDrmTest : public VkLayerTest {
     std::vector<uint64_t> GetFormatModifier(VkFormat format, VkFormatFeatureFlags2 features, uint32_t plane_count = 1);
 };
 
+class MeshTest : public virtual VkLayerTest {
+  public:
+    void InitBasicMeshAndTask();
+};
+
 class QueryTest : public VkLayerTest {
   public:
     bool HasZeroTimestampValidBits();
@@ -355,8 +431,13 @@ class GpuAVRayTracingTest : public GpuAVTest, public RayTracingTest {};
 
 class ShaderObjectTest : public virtual VkLayerTest {
   public:
-    void InitBasicShaderObject();
+    void InitBasicShaderObject(void *instance_pnext = nullptr);
     void InitBasicMeshShaderObject(APIVersion target_api_version);
+
+    // Many tests just need a basic vert/frag shader
+    vkt::Shader m_vert_shader;
+    vkt::Shader m_frag_shader;
+    void CreateMinimalShaders();
 };
 
 class SyncObjectTest : public VkLayerTest {
@@ -368,35 +449,36 @@ class SyncObjectTest : public VkLayerTest {
 #endif
 };
 
-class WsiTest : public VkLayerTest {
+class TensorTest : public VkLayerTest {
   public:
-    // most tests need images in VK_IMAGE_LAYOUT_PRESENT_SRC_KHR layout
-    void SetImageLayoutPresentSrc(VkImage image);
-    VkImageMemoryBarrier TransitionToPresent(VkImage swapchain_image, VkImageLayout old_layout, VkAccessFlags src_access_mask);
-
-  protected:
-#ifdef VK_USE_PLATFORM_WAYLAND_KHR
-    struct WaylandContext {
-        wl_display *display = nullptr;
-        wl_registry *registry = nullptr;
-        wl_surface *surface = nullptr;
-        wl_compositor *compositor = nullptr;
-    };
-    void InitWaylandContext(WaylandContext& context);
-    void ReleaseWaylandContext(WaylandContext& context);
-#endif
+    void InitBasicTensor();
+    static VkTensorDescriptionARM DefaultDesc();
+    static VkTensorDescriptionARM TensorShaderDesc();
+    static VkTensorCreateInfoARM DefaultCreateInfo(VkTensorDescriptionARM *desc = nullptr);
 };
 
-class YcbcrTest : public VkLayerTest {
+class DataGraphTest : public VkLayerTest {
   public:
-    void InitBasicYcbcr(void *pNextFeatures = nullptr);
+    void InitBasicDataGraph();
+    static void CheckSessionMemory(const vkt::DataGraphPipelineSession& session);
+    static std::vector<VkBindDataGraphPipelineSessionMemoryInfoARM> InitSessionBindInfo(const vkt::DataGraphPipelineSession& session, const std::vector<vkt::DeviceMemory>& device_mem);
+    static VkTensorDescriptionARM DefaultDesc();
+    static VkTensorDescriptionARM DefaultConstantTensorDesc();
+    static VkDataGraphPipelineConstantARM GetConstant(const VkTensorDescriptionARM &desc = defaultConstantTensorDesc);
+
+    static const std::string IncorrectSpirvMessage;
+    static const VkTensorDescriptionARM defaultConstantTensorDesc;
+};
+
+class WsiTest : public VkLayerTest {
+  protected:
+    // Find physical device group that contains physical device selected by the test framework
+    std::optional<VkPhysicalDeviceGroupProperties> FindPhysicalDeviceGroup();
 };
 
 class CooperativeMatrixTest : public VkLayerTest {
   public:
     void InitCooperativeMatrixKHR();
-    bool HasValidProperty(VkScopeKHR scope, uint32_t m, uint32_t n, uint32_t k, VkComponentTypeKHR type);
-    std::vector<VkCooperativeMatrixPropertiesKHR> coop_matrix_props;
 };
 
 class ParentTest : public VkLayerTest {
@@ -421,31 +503,9 @@ VKAPI_ATTR VkBool32 VKAPI_CALL DebugUtilsCallback(VkDebugUtilsMessageSeverityFla
                                                   VkDebugUtilsMessageTypeFlagsEXT messageTypes,
                                                   const VkDebugUtilsMessengerCallbackDataEXT *pCallbackData, void *pUserData);
 
-void TestRenderPassCreate(ErrorMonitor *error_monitor, const vkt::Device &device, const VkRenderPassCreateInfo &create_info,
-                          bool rp2_supported, const char *rp1_vuid, const char *rp2_vuid);
-void PositiveTestRenderPassCreate(ErrorMonitor *error_monitor, const vkt::Device &device, const VkRenderPassCreateInfo &create_info,
-                                  bool rp2_supported);
-void PositiveTestRenderPass2KHRCreate(const vkt::Device &device, const VkRenderPassCreateInfo2KHR &create_info);
-void TestRenderPass2KHRCreate(ErrorMonitor &error_monitor, const vkt::Device &device, const VkRenderPassCreateInfo2KHR &create_info,
-                              const std::initializer_list<const char *> &vuids);
-void TestRenderPassBegin(ErrorMonitor *error_monitor, const VkDevice device, const VkCommandBuffer command_buffer,
-                         const VkRenderPassBeginInfo *begin_info, bool rp2Supported, const char *rp1_vuid, const char *rp2_vuid);
-
-VkResult GPDIFPHelper(VkPhysicalDevice dev, const VkImageCreateInfo *ci, VkImageFormatProperties *limits = nullptr);
-
 VkFormat FindFormatWithoutFeatures(VkPhysicalDevice gpu, VkImageTiling tiling,
-                                   VkFormatFeatureFlags undesired_features = vvl::kU32Max);
+                                   VkFormatFeatureFlags undesired_features = vvl::kNoIndex32);
 
 VkFormat FindFormatWithoutFeatures2(VkPhysicalDevice gpu, VkImageTiling tiling, VkFormatFeatureFlags2 undesired_features);
 
-void CreateSamplerTest(VkLayerTest &test, const VkSamplerCreateInfo *pCreateInfo, const std::string &code = "");
-
-void CreateBufferTest(VkLayerTest &test, const VkBufferCreateInfo *pCreateInfo, const std::string &code = "");
-
-void CreateImageTest(VkLayerTest &test, const VkImageCreateInfo *pCreateInfo, const std::string &code = "");
-
-void CreateBufferViewTest(VkLayerTest &test, const VkBufferViewCreateInfo *pCreateInfo, const std::vector<std::string> &codes);
-
-void CreateImageViewTest(VkLayerTest &test, const VkImageViewCreateInfo *pCreateInfo, const std::string &code = "");
-
-void print_android(const char *c);
+void PrintAndroid(const char *c);
