@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2023-2024 Valve Corporation
- * Copyright (c) 2023-2024 LunarG, Inc.
+ * Copyright (c) 2023-2026 Valve Corporation
+ * Copyright (c) 2023-2026 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,9 +21,9 @@ TEST_F(NegativeDeviceQueue, FamilyIndex) {
     RETURN_IF_SKIP(InitFramework());
 
     uint32_t queue_family_count;
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, nullptr);
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, nullptr);
     std::vector<VkQueueFamilyProperties> queue_props(queue_family_count);
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, queue_props.data());
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, queue_props.data());
 
     uint32_t queue_family_index = queue_family_count;
 
@@ -42,7 +42,7 @@ TEST_F(NegativeDeviceQueue, FamilyIndex) {
 
     m_errorMonitor->SetDesiredError("VUID-VkDeviceQueueCreateInfo-queueFamilyIndex-00381");
     VkDevice device;
-    vk::CreateDevice(gpu(), &device_ci, nullptr, &device);
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
     m_errorMonitor->VerifyFound();
 }
 
@@ -55,50 +55,48 @@ TEST_F(NegativeDeviceQueue, FamilyIndexUsage) {
     all_queue_count_ = true;
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
-    VkBufferCreateInfo buffCI = vku::InitStructHelper();
-    buffCI.size = 1024;
-    buffCI.usage = VK_BUFFER_USAGE_TRANSFER_DST_BIT;
-    buffCI.queueFamilyIndexCount = 2;
+    VkBufferCreateInfo buffer_ci = vku::InitStructHelper();
+    buffer_ci.size = 1024;
+    buffer_ci.usage = VK_BUFFER_USAGE_TRANSFER_DST_BIT;
+    buffer_ci.queueFamilyIndexCount = 2;
     // Introduce failure by specifying invalid queue_family_index
     uint32_t qfi[2];
     qfi[0] = 777;
     qfi[1] = 0;
 
-    buffCI.pQueueFamilyIndices = qfi;
-    buffCI.sharingMode = VK_SHARING_MODE_CONCURRENT;  // qfi only matters in CONCURRENT mode
+    buffer_ci.pQueueFamilyIndices = qfi;
+    buffer_ci.sharingMode = VK_SHARING_MODE_CONCURRENT;  // qfi only matters in CONCURRENT mode
 
     // Test for queue family index out of range
-    CreateBufferTest(*this, &buffCI, "VUID-VkBufferCreateInfo-sharingMode-01419");
+    CreateBufferTest(buffer_ci, "VUID-VkBufferCreateInfo-sharingMode-01419");
 
     // Test for non-unique QFI in array
     qfi[0] = 0;
-    CreateBufferTest(*this, &buffCI, "VUID-VkBufferCreateInfo-sharingMode-01419");
+    CreateBufferTest(buffer_ci, "VUID-VkBufferCreateInfo-sharingMode-01419");
 
-    if (m_device->phy().queue_properties_.size() > 2) {
+    if (m_device->Physical().queue_properties_.size() > 2) {
         m_errorMonitor->SetDesiredError("VUID-vkQueueSubmit-pSubmits-04626");
 
         // Create buffer shared to queue families 1 and 2, but submitted on queue family 0
-        buffCI.queueFamilyIndexCount = 2;
+        buffer_ci.queueFamilyIndexCount = 2;
         qfi[0] = 1;
         qfi[1] = 2;
-        vkt::Buffer ib;
-        ib.init(*m_device, buffCI);
+        vkt::Buffer ib(*m_device, buffer_ci);
 
-        m_command_buffer.begin();
-        vk::CmdFillBuffer(m_command_buffer.handle(), ib.handle(), 0, 16, 5);
-        m_command_buffer.end();
-        m_default_queue->Submit(m_command_buffer);
-        m_default_queue->Wait();
+        m_command_buffer.Begin();
+        vk::CmdFillBuffer(m_command_buffer, ib, 0, 16, 5);
+        m_command_buffer.End();
+        m_default_queue->SubmitAndWait(m_command_buffer);
         m_errorMonitor->VerifyFound();
     }
 
     // If there is more than one queue family, create a device with a single queue family, then create a buffer
     // with SHARING_MODE_CONCURRENT that uses a non-device PDEV queue family.
     uint32_t queue_count;
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_count, NULL);
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_count, NULL);
     std::vector<VkQueueFamilyProperties> queue_props;
     queue_props.resize(queue_count);
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_count, queue_props.data());
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_count, queue_props.data());
 
     if (queue_count < 3) {
         GTEST_SKIP() << "Multiple queue families are required to run this test.";
@@ -117,13 +115,13 @@ TEST_F(NegativeDeviceQueue, FamilyIndexUsage) {
 
     // Create a device with a single queue family
     VkDevice second_device;
-    ASSERT_EQ(VK_SUCCESS, vk::CreateDevice(gpu(), &dev_info, nullptr, &second_device));
+    ASSERT_EQ(VK_SUCCESS, vk::CreateDevice(Gpu(), &dev_info, nullptr, &second_device));
 
     // Select Queue family for CONCURRENT buffer that is not owned by device
-    buffCI.queueFamilyIndexCount = 2;
+    buffer_ci.queueFamilyIndexCount = 2;
     qfi[1] = 2;
     VkBuffer buffer = VK_NULL_HANDLE;
-    vk::CreateBuffer(second_device, &buffCI, NULL, &buffer);
+    vk::CreateBuffer(second_device, &buffer_ci, NULL, &buffer);
     vk::DestroyBuffer(second_device, buffer, nullptr);
     vk::DestroyDevice(second_device, nullptr);
 }
@@ -139,9 +137,9 @@ TEST_F(NegativeDeviceQueue, FamilyIndexUnique) {
     VkQueueFamilyProperties queue_properties;  // selected queue family used
     uint32_t queue_family_index = 0;
     uint32_t queue_family_count = 0;
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, nullptr);
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, nullptr);
     std::vector<VkQueueFamilyProperties> queue_families(queue_family_count);
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, queue_families.data());
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, queue_families.data());
 
     for (size_t i = 0; i < queue_families.size(); i++) {
         if (queue_families[i].queueCount > 1) {
@@ -177,7 +175,7 @@ TEST_F(NegativeDeviceQueue, FamilyIndexUnique) {
     device_create_info.enabledExtensionCount = 0;
 
     m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802");
-    vk::CreateDevice(gpu(), &device_create_info, nullptr, &test_device);
+    vk::CreateDevice(Gpu(), &device_create_info, nullptr, &test_device);
     m_errorMonitor->VerifyFound();
 }
 
@@ -190,9 +188,9 @@ TEST_F(NegativeDeviceQueue, MismatchedGlobalPriority) {
     RETURN_IF_SKIP(InitFramework());
 
     uint32_t queue_family_count;
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, nullptr);
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, nullptr);
     std::vector<VkQueueFamilyProperties> queue_props(queue_family_count);
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, queue_props.data());
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, queue_props.data());
 
     uint32_t queue_family_index = queue_family_count;
 
@@ -206,11 +204,11 @@ TEST_F(NegativeDeviceQueue, MismatchedGlobalPriority) {
         GTEST_SKIP() << "Multiple queues from same queue family are required to run this test";
     }
 
-    VkDeviceQueueGlobalPriorityCreateInfoKHR queue_global_priority_ci[2] = {};
+    VkDeviceQueueGlobalPriorityCreateInfo queue_global_priority_ci[2] = {};
     queue_global_priority_ci[0] = vku::InitStructHelper();
-    queue_global_priority_ci[0].globalPriority = VK_QUEUE_GLOBAL_PRIORITY_LOW_KHR;
+    queue_global_priority_ci[0].globalPriority = VK_QUEUE_GLOBAL_PRIORITY_LOW;
     queue_global_priority_ci[1] = vku::InitStructHelper();
-    queue_global_priority_ci[1].globalPriority = VK_QUEUE_GLOBAL_PRIORITY_MEDIUM_KHR;
+    queue_global_priority_ci[1].globalPriority = VK_QUEUE_GLOBAL_PRIORITY_MEDIUM;
 
     float priorities[] = {1.0f, 1.0f};
     VkDeviceQueueCreateInfo device_queue_ci[2] = {};
@@ -234,7 +232,7 @@ TEST_F(NegativeDeviceQueue, MismatchedGlobalPriority) {
     m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-queueFamilyIndex-02802");
     m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-pQueueCreateInfos-06654");
     VkDevice device;
-    vk::CreateDevice(gpu(), &device_ci, nullptr, &device);
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
     m_errorMonitor->VerifyFound();
 }
 
@@ -244,9 +242,9 @@ TEST_F(NegativeDeviceQueue, QueueCount) {
     RETURN_IF_SKIP(InitFramework());
 
     uint32_t queue_family_count;
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, nullptr);
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, nullptr);
     std::vector<VkQueueFamilyProperties> queue_props(queue_family_count);
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, queue_props.data());
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, queue_props.data());
 
     const uint32_t invalid_count = queue_props[0].queueCount + 1;
     std::vector<float> priorities(invalid_count);
@@ -266,7 +264,7 @@ TEST_F(NegativeDeviceQueue, QueueCount) {
 
     VkDevice device = VK_NULL_HANDLE;
     m_errorMonitor->SetDesiredError("VUID-VkDeviceQueueCreateInfo-queueCount-00382");
-    vk::CreateDevice(gpu(), &device_ci, nullptr, &device);
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
     m_errorMonitor->VerifyFound();
 }
 
@@ -276,9 +274,9 @@ TEST_F(NegativeDeviceQueue, QueuePriorities) {
     RETURN_IF_SKIP(InitFramework());
 
     uint32_t queue_family_count;
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, nullptr);
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, nullptr);
     std::vector<VkQueueFamilyProperties> queue_props(queue_family_count);
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &queue_family_count, queue_props.data());
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &queue_family_count, queue_props.data());
 
     VkDeviceQueueCreateInfo device_queue_ci = vku::InitStructHelper();
     device_queue_ci.queueFamilyIndex = 0;
@@ -296,13 +294,13 @@ TEST_F(NegativeDeviceQueue, QueuePriorities) {
     const float priority_high = 2.0f;
     device_queue_ci.pQueuePriorities = &priority_high;
     m_errorMonitor->SetDesiredError("VUID-VkDeviceQueueCreateInfo-pQueuePriorities-00383");
-    vk::CreateDevice(gpu(), &device_ci, nullptr, &device);
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
     m_errorMonitor->VerifyFound();
 
     const float priority_low = -1.0f;
     device_queue_ci.pQueuePriorities = &priority_low;
     m_errorMonitor->SetDesiredError("VUID-VkDeviceQueueCreateInfo-pQueuePriorities-00383");
-    vk::CreateDevice(gpu(), &device_ci, nullptr, &device);
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
     m_errorMonitor->VerifyFound();
 }
 
@@ -312,7 +310,7 @@ TEST_F(NegativeDeviceQueue, BindPipeline) {
     InitRenderTarget();
 
     uint32_t only_transfer_queueFamilyIndex = vvl::kU32Max;
-    const auto q_props = m_device->phy().queue_properties_;
+    const auto q_props = m_device->Physical().queue_properties_;
     for (uint32_t i = 0; i < (uint32_t)q_props.size(); i++) {
         if (q_props[i].queueFlags == VK_QUEUE_TRANSFER_BIT) {
             only_transfer_queueFamilyIndex = i;
@@ -331,83 +329,54 @@ TEST_F(NegativeDeviceQueue, BindPipeline) {
     c_pipe.CreateComputePipeline();
 
     // Get implicit VU because using Transfer only instead of a Graphics-only or Compute-only queue
-    commandBuffer.begin();
+    commandBuffer.Begin();
     m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-commandBuffer-cmdpool");
     m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-pipelineBindPoint-00777");
-    vk::CmdBindPipeline(commandBuffer.handle(), VK_PIPELINE_BIND_POINT_COMPUTE, c_pipe.Handle());
+    vk::CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_COMPUTE, c_pipe);
     m_errorMonitor->VerifyFound();
 
     m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-commandBuffer-cmdpool");
     m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-pipelineBindPoint-00778");
-    vk::CmdBindPipeline(commandBuffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, g_pipe.Handle());
+    vk::CmdBindPipeline(commandBuffer, VK_PIPELINE_BIND_POINT_GRAPHICS, g_pipe);
     m_errorMonitor->VerifyFound();
-    commandBuffer.end();
+    commandBuffer.End();
 }
 
 TEST_F(NegativeDeviceQueue, CreateCommandPool) {
     TEST_DESCRIPTION("vkCreateCommandPool with bad queue");
     RETURN_IF_SKIP(Init());
-    const size_t queue_count = m_device->phy().queue_properties_.size();
+    const size_t queue_count = m_device->Physical().queue_properties_.size();
     m_errorMonitor->SetDesiredError("VUID-vkCreateCommandPool-queueFamilyIndex-01937");
     vkt::CommandPool commandPool(*m_device, queue_count + 1);
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeDeviceQueue, Robustness2WithoutRobustness) {
-    AddRequiredExtensions(VK_EXT_ROBUSTNESS_2_EXTENSION_NAME);
-    RETURN_IF_SKIP(Init());
-
-    float priority = 1.0f;
-    VkDeviceQueueCreateInfo device_queue_ci = vku::InitStructHelper();
-    device_queue_ci.queueFamilyIndex = 0u;
-    device_queue_ci.queueCount = 1u;
-    device_queue_ci.pQueuePriorities = &priority;
-
-    VkPhysicalDeviceRobustness2FeaturesEXT robustness_2_features = vku::InitStructHelper();
-    GetPhysicalDeviceFeatures2(robustness_2_features);
-
-    if (!robustness_2_features.robustBufferAccess2) {
-        GTEST_SKIP() << "robustBufferAccess2 not supported.";
-    }
-
-    VkPhysicalDeviceFeatures2 features2 = vku::InitStructHelper(&robustness_2_features);
-
-    VkDeviceCreateInfo device_ci = vku::InitStructHelper(&features2);
-    device_ci.queueCreateInfoCount = 1u;
-    device_ci.pQueueCreateInfos = &device_queue_ci;
-    device_ci.enabledLayerCount = 0u;
-    device_ci.enabledExtensionCount = m_device_extension_names.size();
-    device_ci.ppEnabledExtensionNames = m_device_extension_names.data();
-
-    VkDevice device;
-    m_errorMonitor->SetDesiredError("VUID-VkPhysicalDeviceRobustness2FeaturesEXT-robustBufferAccess2-04000");
-    vk::CreateDevice(m_device->phy().handle(), &device_ci, nullptr, &device);
-    m_errorMonitor->VerifyFound();
-}
-
 TEST_F(NegativeDeviceQueue, QueuesSameQueueFamily) {
     TEST_DESCRIPTION("Request more queues than available from queue family");
 
     SetTargetApiVersion(VK_API_VERSION_1_1);
     RETURN_IF_SKIP(InitFramework());
     VkPhysicalDeviceProtectedMemoryFeatures protected_memory_features = vku::InitStructHelper();
+    auto features2 = GetPhysicalDeviceFeatures2(protected_memory_features);
     if (!protected_memory_features.protectedMemory) {
         GTEST_SKIP() << "protectedMemory not supported";
     }
-    RETURN_IF_SKIP(InitState(nullptr, &protected_memory_features));
 
     uint32_t qf_count;
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &qf_count, nullptr);
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &qf_count, nullptr);
     std::vector<VkQueueFamilyProperties> qf_props(qf_count);
-    vk::GetPhysicalDeviceQueueFamilyProperties(gpu(), &qf_count, qf_props.data());
+    vk::GetPhysicalDeviceQueueFamilyProperties(Gpu(), &qf_count, qf_props.data());
 
-    uint32_t index = 0;
+    uint32_t index = UINT32_MAX;
     for (uint32_t i = 0; i < qf_count; ++i) {
-        if (qf_props[i].queueFlags & VK_DEVICE_QUEUE_CREATE_PROTECTED_BIT) {
+        if (qf_props[i].queueFlags & VK_QUEUE_PROTECTED_BIT) {
             index = i;
             break;
         }
     }
+    if (UINT32_MAX == index) {
+        GTEST_SKIP() << "no queue supports protected memory";
+    }
 
     std::vector<float> priorities(qf_props[index].queueCount, 1.0f);
     VkDeviceQueueCreateInfo device_queue_ci[2];
@@ -421,13 +390,13 @@ TEST_F(NegativeDeviceQueue, QueuesSameQueueFamily) {
     device_queue_ci[1].queueCount = 1u;
     device_queue_ci[1].pQueuePriorities = priorities.data();
 
-    VkDeviceCreateInfo device_ci = vku::InitStructHelper();
+    VkDeviceCreateInfo device_ci = vku::InitStructHelper(&features2);
     device_ci.queueCreateInfoCount = 2u;
     device_ci.pQueueCreateInfos = device_queue_ci;
 
     VkDevice device;
     m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-pQueueCreateInfos-06755");
-    vk::CreateDevice(gpu(), &device_ci, nullptr, &device);
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
     m_errorMonitor->VerifyFound();
 }
 
@@ -439,7 +408,7 @@ TEST_F(NegativeDeviceQueue, MismatchedQueueFamiliesOnSubmit) {
     RETURN_IF_SKIP(Init());  // assumes it initializes all queue families on vk::CreateDevice
 
     // This test is meaningless unless we have multiple queue families
-    auto queue_family_properties = m_device->phy().queue_properties_;
+    auto queue_family_properties = m_device->Physical().queue_properties_;
     std::vector<uint32_t> queue_families;
     for (uint32_t i = 0; i < queue_family_properties.size(); ++i)
         if (queue_family_properties[i].queueCount > 0) queue_families.push_back(i);
@@ -457,8 +426,8 @@ TEST_F(NegativeDeviceQueue, MismatchedQueueFamiliesOnSubmit) {
     vkt::CommandPool cmd_pool(*m_device, queue_family);
     vkt::CommandBuffer cmd_buff(*m_device, cmd_pool);
 
-    cmd_buff.begin();
-    cmd_buff.end();
+    cmd_buff.Begin();
+    cmd_buff.End();
 
     // Submit on the wrong queue
     VkSubmitInfo submit_info = vku::InitStructHelper();
@@ -469,3 +438,119 @@ TEST_F(NegativeDeviceQueue, MismatchedQueueFamiliesOnSubmit) {
     vk::QueueSubmit(other_queue, 1, &submit_info, VK_NULL_HANDLE);
     m_errorMonitor->VerifyFound();
 }
+
+TEST_F(NegativeDeviceQueue, DeviceCreateInvalidParameters) {
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    RETURN_IF_SKIP(InitFramework());
+
+    float priority = 1.0f;
+    VkDeviceQueueCreateInfo device_queue_ci = vku::InitStructHelper();
+    device_queue_ci.queueFamilyIndex = 0u;
+    device_queue_ci.queueCount = 1;
+    device_queue_ci.pQueuePriorities = &priority;
+
+    VkPhysicalDeviceVulkan11Features features11 = vku::InitStructHelper();
+    VkPhysicalDeviceVulkan11Features features11_duplicated = vku::InitStructHelper(&features11);
+
+    VkDeviceCreateInfo device_ci = vku::InitStructHelper(&features11_duplicated);
+    device_ci.queueCreateInfoCount = 1u;
+    device_ci.pQueueCreateInfos = &device_queue_ci;
+    device_ci.enabledLayerCount = 0u;
+    device_ci.enabledExtensionCount = m_device_extension_names.size();
+    device_ci.ppEnabledExtensionNames = m_device_extension_names.data();
+
+    VkDevice device;
+    m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-sType-unique");
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
+    m_errorMonitor->VerifyFound();
+
+    device_ci.pNext = nullptr;
+    device_ci.flags = 1u;
+    m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-flags-zerobitmask");
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
+    m_errorMonitor->VerifyFound();
+
+    device_ci.flags = 0u;
+    device_ci.pQueueCreateInfos = nullptr;
+    m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-pQueueCreateInfos-parameter");
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
+    m_errorMonitor->VerifyFound();
+
+    if (!DeviceExtensionSupported(VK_KHR_MAINTENANCE_9_EXTENSION_NAME)) {
+        device_ci.pQueueCreateInfos = &device_queue_ci;
+        device_ci.queueCreateInfoCount = 0u;
+        m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-None-10778");
+        vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
+        m_errorMonitor->VerifyFound();
+    }
+}
+
+TEST_F(NegativeDeviceQueue, NoQueues) {
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    AddRequiredExtensions(VK_KHR_MAINTENANCE_9_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::maintenance9);
+    RETURN_IF_SKIP(InitFramework());
+
+    VkDeviceCreateInfo device_ci = vku::InitStructHelper();
+    device_ci.pNext = requested_features_.GetEnabledFeatures2();
+    device_ci.enabledExtensionCount = static_cast<uint32_t>(m_device_extension_names.size());
+    device_ci.ppEnabledExtensionNames = m_device_extension_names.data();
+
+    vkt::Device device(gpu_, device_ci);
+
+    VkBufferCreateInfo buffer_create_info = vku::InitStructHelper();
+    buffer_create_info.size = 4096;
+    buffer_create_info.usage = VK_BUFFER_USAGE_TRANSFER_SRC_BIT;
+    VkBuffer buffer;
+    m_errorMonitor->SetDesiredError("VUID-vkCreateBuffer-device-queuecount");
+    vk::CreateBuffer(device, &buffer_create_info, nullptr, &buffer);
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeDeviceQueue, DeviceCreateEnabledLayerNamesPointer) {
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    RETURN_IF_SKIP(InitFramework());
+
+    float priority = 1.0f;
+    VkDeviceQueueCreateInfo device_queue_ci = vku::InitStructHelper();
+    device_queue_ci.queueFamilyIndex = 0u;
+    device_queue_ci.queueCount = 1;
+    device_queue_ci.pQueuePriorities = &priority;
+
+    VkDevice device;
+    VkDeviceCreateInfo device_ci = vku::InitStructHelper();
+    device_ci.queueCreateInfoCount = 1u;
+    device_ci.pQueueCreateInfos = &device_queue_ci;
+    device_ci.enabledExtensionCount = m_device_extension_names.size();
+    device_ci.ppEnabledExtensionNames = m_device_extension_names.data();
+    device_ci.enabledLayerCount = 1u;
+    device_ci.ppEnabledLayerNames = nullptr;
+    m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-ppEnabledLayerNames-parameter");
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
+    m_errorMonitor->VerifyFound();
+}
+
+// Test works, but needs Loader fix in 369afe24d1351d6e03cbfc3daf1fc5f6cd103649
+// Enable once enough CI machines have loader patch
+TEST_F(NegativeDeviceQueue, DISABLED_DeviceCreateEnabledExtensionNamesPointer) {
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    RETURN_IF_SKIP(InitFramework());
+
+    float priority = 1.0f;
+    VkDeviceQueueCreateInfo device_queue_ci = vku::InitStructHelper();
+    device_queue_ci.queueFamilyIndex = 0u;
+    device_queue_ci.queueCount = 1;
+    device_queue_ci.pQueuePriorities = &priority;
+
+    VkDevice device;
+    VkDeviceCreateInfo device_ci = vku::InitStructHelper();
+    device_ci.queueCreateInfoCount = 1u;
+    device_ci.pQueueCreateInfos = &device_queue_ci;
+    device_ci.enabledLayerCount = 0u;
+    device_ci.ppEnabledLayerNames = nullptr;
+    device_ci.enabledExtensionCount = 1u;
+    device_ci.ppEnabledExtensionNames = nullptr;
+    m_errorMonitor->SetDesiredError("VUID-VkDeviceCreateInfo-ppEnabledExtensionNames-parameter");
+    vk::CreateDevice(Gpu(), &device_ci, nullptr, &device);
+    m_errorMonitor->VerifyFound();
+}
